# -*- coding: UTF-8 -*-

from lava.api.util import LavaError


class ByteCodeError(LavaError):

    UNEXPECTED = "Something unexpected happened"

    def __init__(self, message):
        super(ByteCodeError, self).__init__(message)

    @classmethod
    def unexpected(cls):
        return cls(cls.UNEXPECTED)



# -*- coding: UTF-8 -*-

__initialized = False

try:
    from vulkan import *
    from vulkan import __version__
    import vulkan._vulkan as _vulkan
    __initialized = True

except OSError:
    # patch constants from vulkan package which are needed for the modules to be importable
    VK_PHYSICAL_DEVICE_TYPE_INTEGRATED_GPU = 1
    VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU = 2
    VK_PHYSICAL_DEVICE_TYPE_VIRTUAL_GPU = 3
    VK_PHYSICAL_DEVICE_TYPE_CPU = 4

    VK_QUEUE_GRAPHICS_BIT = 0x00000001
    VK_QUEUE_COMPUTE_BIT = 0x00000002
    VK_QUEUE_TRANSFER_BIT = 0x00000004
    VK_QUEUE_SPARSE_BINDING_BIT = 0x00000008

    VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT = 0x00000001
    VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT = 0x00000002
    VK_MEMORY_PROPERTY_HOST_COHERENT_BIT = 0x00000004
    VK_MEMORY_PROPERTY_HOST_CACHED_BIT = 0x00000008
    VK_MEMORY_PROPERTY_LAZILY_ALLOCATED_BIT = 0x00000010

    VK_BUFFER_USAGE_TRANSFER_SRC_BIT = 0x00000001
    VK_BUFFER_USAGE_TRANSFER_DST_BIT = 0x00000002
    VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT = 0x00000010
    VK_BUFFER_USAGE_STORAGE_BUFFER_BIT = 0x00000020

    VK_DESCRIPTOR_TYPE_SAMPLER = 0
    VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER = 6
    VK_DESCRIPTOR_TYPE_STORAGE_BUFFER = 7

    VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT = 0x00000001
    VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT = 0x00000004


def initialized():
    return __initialized



# -*- coding: UTF-8 -*-

import lava.api.vulkan as vk
from lava.api.bytecode.logical import ByteCode
from lava.api.bytecode.physical import ByteCodeData
from lava.api.util import Destroyable


class Shader(Destroyable):

    def __init__(self, device, bytez, entry_point=None):
        super(Shader, self).__init__()
        self.device = device
        self.handle = vk.vkCreateShaderModule(self.device.handle, vk.VkShaderModuleCreateInfo(codeSize=len(bytez),
                                                                                              pCode=bytez), None)
        self.byte_code_data = ByteCodeData(bytez)
        self.byte_code = None

        self.entry_point, self.entry_point_index = ByteCode.check_entry_point(self.byte_code_data, entry_point)
        self.local_size = ByteCode.check_local_size(self.byte_code_data, self.entry_point_index)

    @classmethod
    def from_file(cls, device, path, entry_point=None):
        with open(path, "rb") as f:
            bytez = f.read()
        return cls(device, bytez, entry_point)

    def inspect(self):
        self.byte_code = ByteCode(self.byte_code_data, self.entry_point)

    @property
    def code(self):
        if self.byte_code is None:
            raise RuntimeError("Shader bytecode was not inspected yet")
        return self.byte_code

    def get_entry_point(self):
        return self.entry_point

    def get_local_size(self):
        return self.local_size

    def _destroy(self):
        vk.vkDestroyShaderModule(self.device.handle, self.handle, None)
